// m1.c: Steve Ellingson, Virginia Tech, 2010 Nov 05
// ---
// Reads file output from GSM, which is T [K] in HEALpix NSIDE=512 format
// Writes it back out as l [rad], b [rad], T [K] where (l,b) are Galactic coordinates
// ---
// USAGE:
// $ ./m1 <gsm_output_filename>
// output goes to m1.dat

#include <math.h>
#include <stdio.h>
#include <string.h>

#define NSIDE 512

#define PI 3.14159265

#define ME "m1"

main ( int narg, char *argv[] ) {

  long int npix;

  char gsm_filename[256];

  FILE *fpi;
  long int p=0;
  float x;

  int eLat=0; 

  long int pp,i,j;
  float ph;
  int s;
  float z,theta,phi;

  float l,b;

  FILE *fpo;

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  printf("%s: Starting\n",ME);
  printf("%s: Compiled for N_{side} = %d\n",ME,NSIDE);

  /* intializing */
  npix = 12*NSIDE*NSIDE;
  printf("%s: N_{pix} = %ld\n",ME,npix);

  /* clearing this just in case */
  sprintf(gsm_filename,"");

  if (narg>1) {
      sscanf(argv[1],"%s",gsm_filename);
    } else {
      printf("%s: FATAL: gsm_filename not specified\n",ME);
      return;
    }
  printf("%s: gsm_filename = '%s'\n",ME,gsm_filename);

  fpi = fopen(gsm_filename,"r");
  if (!fpi) {
    printf("%s: FATAL: Couldn't open '%s'\n",ME,gsm_filename);
    return;
    }

  fpo = fopen("m1.dat","w");
  if (!fpo) {
    printf("%s: FATAL: Couldn't open m1.dat\n",ME);
    return;
    }

  while (fscanf(fpi,"%f",&x)!=EOF) {

    /* figure out where we are */
    switch (eLat) {
      case 0: /* North Polar Cap */
        ph = (p+1)/2.0;
        i = floor(sqrt(ph-sqrt(floor(ph)))) + 1; /* ring index */
        j = p + 1 - (2*i*(i-1));                 /* pixel-in-ring index */
        z = 1.0 - ((float)(i*i))/(3.0*NSIDE*NSIDE);  /* cos(theta) */
        s = 1;
        phi = PI*(j-(s/2.0))/(2.0*i);
        if ( (i==(NSIDE-1)) && (j==(4*i)) ) { eLat++;  }
        break;
      case 1: /* North Equitorial Belt */
        pp = p - (2*NSIDE*(NSIDE-1));
        i = floor(pp/(4.0*NSIDE)) + NSIDE;
        j = ( pp % (4*NSIDE) ) + 1;
        z = (4.0/3.0) - ((2.0*i)/(3.0*NSIDE));
        s = (i-NSIDE+1) % 2;
        phi = PI*(j-(s/2.0))/(2.0*NSIDE);
        if ( (i==(2*NSIDE)) && (j==(4*NSIDE)) ) { eLat++;  }
        break;
      case 2: /* South Equitorial Belt */
        pp = p - (2*NSIDE*(NSIDE-1));
        i = floor(pp/(4.0*NSIDE)) + NSIDE;
        j = ( pp % (4*NSIDE) ) + 1;
        z = (4.0/3.0) - ((2.0*i)/(3.0*NSIDE));
        s = (i-NSIDE+1) % 2;
        phi = PI*(j-(s/2.0))/(2.0*NSIDE);
        if ( (i==(3*NSIDE)) && (j==(4*NSIDE)) ) { eLat++;  }
        break;
      case 3: /* South Polar Cap */
        pp = (npix-1)-p;
        ph = (pp+1)/2.0;
        i = floor(sqrt(ph-sqrt(floor(ph)))) + 1; /* ring index */
        j = pp + 1 - (2*i*(i-1));                 /* pixel-in-ring index */
        z = -(1.0 - ((float)(i*i))/(3.0*NSIDE*NSIDE));  /* cos(theta) */
        s = 1;
        phi = (2.0*PI) - (PI*(j-(s/2.0))/(2.0*i));
        if ( p==(npix-1) ) eLat++;
        break;
      default:
        break;
      }

    theta = acos(z);

    // theta {0->180} corresponds to b {+90->-90}
    b = (PI/2.0) - theta; /* radians */
    //// theta {0->180} corresponds to b {-90->+90}
    //b = theta - (PI/2.0); /* radians */

    //// phi {0->360} corresponds to l {-180->+180}
    //l = phi - PI;  /* radians */
    // phi {0->360} corresponds to l {0->+360}
    l = phi;  /* radians */

    /* write it */
    fprintf(fpo,"%f %f %f\n",l,b,x);

    p++;
    }
  printf("%s: Read %ld lines (pixels)\n",ME,p);

  fclose(fpo);
  fclose(fpi);

  return;
  } /* main() */


