// m2.c: Steve Ellingson, Virginia Tech, 2010 Nov 05
// ---
// Reads sky temp file having format l [rad], b [rad], T [K] where (l,b) are Galactic coordinates
//   (presumed to be from m1; i.e., m1.dat)
// Does a very crude 1-deg x 1-deg regridding, and writes output back in a Gnuplot-friendly format
//   in which each line consists of T's at a constant b, and the columns are increasing l.
//   b goes from -90 to +89 degrees, l goes from 0 to 359 degrees.
//   The Gnuplot script m2.gp is provided to display the result. 
//     (From the Gnuplot command prompt, do "load 'm2.gp'")
// The purpose of this code is simply to sanity-check the output of GSM | m1.  This is not 
//   recommended for other uses, as the regridding is very crude.
// ---
// USAGE:
//   m2 <input_filename>
//   output goes to m2.dat
// EXAMPLE: Typical usage will be:
//   $ ./m2 m1.dat

#include <math.h>
#include <stdio.h>
#include <string.h>

#define PI 3.14159265

#define ME "m2"

main ( int narg, char *argv[] ) {

  long int npix;

  char in_filename[256];

  FILE *fpi;
  float l,b,x;

  int il;
  int ib;
  float map[180][360];

  FILE *fpo;

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  printf("%s: Starting\n",ME);

  /* intializing */
  for (ib=0; ib<180; ib++) {
    for (il=0; il<360; il++) {
      map[ib][il] = 0.0;
      }
    }

  /* clearing this just in case */
  sprintf(in_filename,"");

  if (narg>1) {
      sscanf(argv[1],"%s",in_filename);
    } else {
      printf("%s: FATAL: in_filename not specified\n",ME);
      return;
    }
  printf("%s: in_filename = '%s'\n",ME,in_filename);

  fpi = fopen(in_filename,"r");
  if (!fpi) {
    printf("%s: FATAL: Couldn't open '%s'\n",ME,in_filename);
    return;
    }

  while (!feof(fpi)) {

    fscanf(fpi,"%f %f %f",&l,&b,&x);

    ib = ((int) floor(b*180/PI)) + 90;  // Converts {-90:+90} to {0:179}
      if (ib<0) ib=0;
      if (ib>179) ib=179;
    il = ((int) floor(l*180/PI)); 
      if (il<0) il=0;
      if (il>359) il=359;
    map[ib][il] = x;

    //printf("%f %d %f %d %f \n",b*180/PI,ib,l*180/PI,il,x);

    }

  fclose(fpi);

  fpo = fopen("m2.dat","w");
  if (!fpo) {
    printf("%s: FATAL: Couldn't open m2.dat\n",ME);
    return;
    }

  for (ib=0; ib<180; ib++) {
    for (il=0; il<360; il++) {
      fprintf(fpo,"%f ",map[ib][il]);
      }
    fprintf(fpo,"\n");
    }

  fclose(fpo);

  return;
  } /* main() */


